﻿#region Copyright (C) 2003-2015 Stimulsoft
/*
{*******************************************************************}
{																	}
{	Stimulsoft Reports.Net											}
{	Stimulsoft.Report Library										}
{																	}
{	Copyright (C) 2003-2015 Stimulsoft     							}
{	ALL RIGHTS RESERVED												}
{																	}
{	The entire contents of this file is protected by U.S. and		}
{	International Copyright Laws. Unauthorized reproduction,		}
{	reverse-engineering, and distribution of all or any portion of	}
{	the code contained in this file is strictly prohibited and may	}
{	result in severe civil and criminal penalties and will be		}
{	prosecuted to the maximum extent possible under the law.		}
{																	}
{	RESTRICTIONS													}
{																	}
{	THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES			}
{	ARE CONFIDENTIAL AND PROPRIETARY								}
{	TRADE SECRETS OF Stimulsoft										}
{																	}
{	CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON		}
{	ADDITIONAL RESTRICTIONS.										}
{																	}
{*******************************************************************}
*/
#endregion Copyright (C) 2003-2015 Stimulsoft

using System;
using System.ComponentModel;
using System.Collections;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Data;
using Stimulsoft.Base;
using Stimulsoft.Base.Drawing;
using Stimulsoft.Base.Localization;
using Stimulsoft.Base.Services;
using Devart.Data.Oracle;

namespace Stimulsoft.Database
{
    [StiServiceBitmap(typeof(Stimulsoft.Report.Dictionary.StiDataAdapterService), "Stimulsoft.Report.Bmp.DataAdapter.bmp")]
    [StiServiceCategoryBitmap(typeof(Stimulsoft.Report.Dictionary.StiDataAdapterService), "Stimulsoft.Report.Bmp.DataAdapter.bmp")]
    public class StiDotConnectOracleMetadataProvider : BaseMetadataProvider
    {
        #region StiService override
        /// <summary>
        /// Gets a service name.
        /// </summary>
        public override string ServiceName
        {
            get
            {
                return "dotConnectOracle Metadata Provider";
            }
        }
        #endregion

        #region Properties
        [Browsable(true)]
        public OracleConnection Connection
        {
            get
            {
                return internalConnection as OracleConnection;
            }
            set
            {
                if (internalConnection != value)
                {
                    internalConnection = value;
                }
            }
        }

        public override bool IsConnected
        {
            get
            {
                if (Connection != null)
                {
                    return (Connection.State & ConnectionState.Open) == ConnectionState.Open;
                }
                else return false;
            }
            set
            {
                base.IsConnected = value;
            }
        }

        public override bool IsCanCreateInternalConnection
        {
            get
            {
                return true;
            }
        }
        #endregion

        #region Methods

        protected override void DoConnect()
        {
            base.DoConnect();

            CheckConnectionSet();

            try
            {
                Connection.Open();
            }
            catch
            {
                throw;
            }
        }

        protected override void DoDisconnect()
        {
            base.DoDisconnect();

            CheckConnectionSet();
            Connection.Close();
        }

        protected override void CheckConnectionSet()
        {
            if (Connection == null)
            {
                throw new Exception(String.Format(StiLocalization.Get("QueryBuilder", "NoConnectionObject"), "Connection"));
            }
        }

        protected override IDataReader PrepareSQLDatasetInternal(string sql, bool schemaOnly)
        {
            OracleCommand command = null;
            OracleDataReader reader = null;

            if (!IsConnected) Connect();

            command = Connection.CreateCommand();
            command.CommandText = sql;

            if (schemaOnly)
            {
                reader = command.ExecuteReader(CommandBehavior.SchemaOnly);
            }
            else
            {
                reader = command.ExecuteReader();
            }

            return reader;
        }

        protected override void ExecSQLInternal(string sql)
        {
            if (!IsConnected) Connect();

            OracleCommand command = Connection.CreateCommand();
            command.CommandText = sql;
            command.ExecuteNonQuery();
        }


        public override void CreateAndBindInternalConnectionObj()
        {
            base.CreateAndBindInternalConnectionObj();
            Connection = new OracleConnection();
            AddInternalConnectionObject(Connection);
        }
        #endregion

        public StiDotConnectOracleMetadataProvider()
            : this(null)
        {
        }

        public StiDotConnectOracleMetadataProvider(IDbConnection connection)
            : base(connection)
        {
        }

        static StiDotConnectOracleMetadataProvider()
        {
            Stimulsoft.Database.Helpers.MetadataProviderList.RegisterMetadataProvider(typeof(StiDotConnectOracleMetadataProvider));
        }
    }
}


